<?php
namespace Teknisa\MAN\Factories;

use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class RelatoriosRNFFactory {
    protected $connection;

    public function __construct(Connection $connection) {
        $this->connection = $connection;
    }

    public function findValidacoesRNF($params) {
        try {
            // executando a procedure
            $this->connection->beginTransaction();
            
            // cria a procedure
            $procedure = new StoredProcedure($this->connection, 'VALIDACOES_RNF');
            
            // coloca os parametros
            $procedure->addParam(new Param('P_CDFILIAL',   Param::PARAM_INPUT, $params['P_CDFILIAL'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $params['P_CDOPERADOR'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL',  Param::PARAM_INPUT, $params['P_DTINICIAL'],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL',    Param::PARAM_INPUT, $params['P_DTFINAL'],    Param::PARAM_TYPE_STR));
            
            // executa a procedure
            $procedure->execute();
            $this->connection->commit();
            
            // cria a consulta para o relatório
            $sql = "
                SELECT DSVALIDACAO
                    FROM VALIDACAORNF
                ORDER BY 1
            ";
    
            $std = $this->connection->prepare($sql);
            $std->execute();
            
            return $std->fetchAll();
        } catch(\Exception $e){
            $this->connection->rollback();
            throw $e;
        }
    }

    public function findResumoNotasFiscais($params) {
        $sql = "
            SELECT *
                FROM (
                    SELECT :P_DTINICIAL AS DTINICIAL, :P_DTFINAL AS DTFINAL,
                            F.CDFILIAL, F.NMFILIAL, F.CDFILIALEXT, H.DTA_RNF,
                            B.NUM_CNPJ_CPF, B.NUM_NOTA, B.COD_SERIE,
                            DECODE(H.DTA_IMPORTACAO, NULL, 'P', DECODE(H.DTA_PROCESS, NULL, 'I', 'R')) AS IDSTATUS
                        FROM FILIAL F, HEADERINTSAPFOS H, BODYINTSAPFOS B, (
                            SELECT DISTINCT F.CDFILIAL
                                FROM V_FILIALLOJACAIXA F, FILIOPER FO
                                WHERE ((INSTR(:P_CDFILIAL, F.CDFILIAL) > 0) OR (:P_CDFILIAL = 'T'))
                                    AND FO.CDOPERADOR = :P_CDOPERADOR
                                    AND F.CDFILIAL = FO.CDFILIAL
                        ) FO
                        WHERE FO.CDFILIAL = F.CDFILIAL
                            AND H.COD_UNIDADE_NL = F.CDFILIALEXT
                            AND H.DTA_RNF BETWEEN :P_DTINICIAL AND :P_DTFINAL
                            AND H.NUM_SEQ_HEADER = B.NUM_SEQ_HEADER
                            AND ISNUMBER(F.CDFILIALEXT) = 'S'
                        GROUP BY F.CDFILIAL, F.NMFILIAL, F.CDFILIALEXT, H.DTA_RNF,
                                    B.NUM_CNPJ_CPF, B.NUM_NOTA, B.COD_SERIE,
                                    DECODE(H.DTA_IMPORTACAO, NULL, 'P', DECODE(H.DTA_PROCESS, NULL, 'I', 'R'))
                ) RNF
                WHERE INSTR(:P_IDSTATUS, IDSTATUS) > 0
            ORDER BY CDFILIAL, NMFILIAL, CDFILIALEXT, DTA_RNF, NUM_CNPJ_CPF, NUM_NOTA, COD_SERIE, IDSTATUS
        ";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function findDetalhamentoRNF($params) {
        $sql = "
        SELECT F.CDFILIALEXT, F.NMFILIAL, P.CDCLIENTE, CL.NMRAZSOCCLIE,
                B.NUM_NOTA, B.COD_SERIE, B.DTA_EMISSAO, 'Z' AS TIPO_NOTA,
                B.NUM_CNPJ_CPF,
                DECODE(H.DTA_IMPORTACAO, NULL, 'P', DECODE(H.DTA_PROCESS, NULL, 'I', 'R')) AS IDSTATUS,
                H.NUM_RNF, B.NUM_DOC_CONT, B.NUM_DOC_FISC,
                H.DTA_RNF, I.COD_ITEM_NL, P.NMPRODUTO,
                SUBSTR(I.COD_NBM, 1, 4)||'.'||SUBSTR(I.COD_NBM, 5, 2)||'.'||SUBSTR(I.COD_NBM, 7, 8) AS NCM_EDITADO,
                I.TIP_DOCUMENTO,
                NVL(I.VLR_ITEM, 0) - NVL(I.VLR_DESCONTO, 0) + NVL(I.VLR_FRETE, 0) + NVL(I.VLR_SEGURO, 0) + NVL(I.VLR_OUTRAS_DESP, 0) AS VRPRODUTOS,
                NVL(I.VLR_ITEM, 0) - NVL(I.VLR_DESCONTO, 0) + NVL(I.VLR_FRETE, 0) + NVL(I.VLR_SEGURO, 0) + NVL(I.VLR_OUTRAS_DESP, 0) AS VROPERACAO,
                IMPICMS.VLR_IMPOSTO AS VRICMS, IMPPIS.VLR_IMPOSTO AS VRPIS, 
                IMPCOFINS.VLR_IMPOSTO AS VRCOFINS, 0 AS VLR_OUTRAS,
                H.NUM_SEQ_HEADER, B.COD_OPER, I.NUM_CFOP, I.VLR_UNITARIO, I.QTD_ITEM
            FROM FILIAL F, HEADERINTSAPFOS H, BODYINTSAPFOS B, ITEMINTSAPFOS I, PRODUTO P,
                PARAVEND P 
                    LEFT JOIN CLIENTE CL
                        ON CL.CDCLIENTE = P.CDCLIENTE,
                    (
                    SELECT NUM_SEQ_HEADER, NUM_SEQ_CORPO, NUM_SEQ_ITEM, VLR_IMPOSTO
                        FROM IMPOSTOINTSAPFOS
                        WHERE TIP_IMPOSTO = 'ICMS'
                ) IMPICMS,
                (
                    SELECT NUM_SEQ_HEADER, NUM_SEQ_CORPO, NUM_SEQ_ITEM, VLR_IMPOSTO
                        FROM IMPOSTOINTSAPFOS
                        WHERE TIP_IMPOSTO = 'COFINS'
                ) IMPCOFINS,
                (
                    SELECT NUM_SEQ_HEADER, NUM_SEQ_CORPO, NUM_SEQ_ITEM, VLR_IMPOSTO
                        FROM IMPOSTOINTSAPFOS
                        WHERE TIP_IMPOSTO = 'PIS'
                ) IMPPIS,
                (
                    SELECT DISTINCT F.CDFILIAL
                        FROM V_FILIALLOJACAIXA F, FILIOPER FO
                        WHERE ((INSTR(:P_CDFILIAL, F.CDFILIAL) > 0) OR (:P_CDFILIAL = 'T'))
                            AND FO.CDOPERADOR = :P_CDOPERADOR
                            AND F.CDFILIAL = FO.CDFILIAL
                ) FO
            WHERE FO.CDFILIAL = F.CDFILIAL
                AND H.COD_UNIDADE_NL = F.CDFILIALEXT
                AND H.DTA_RNF BETWEEN :P_DTINICIAL AND :P_DTFINAL
                AND H.NUM_SEQ_HEADER = B.NUM_SEQ_HEADER
                AND ISNUMBER(F.CDFILIALEXT) = 'S'
                AND FO.CDFILIAL = P.CDFILIAL
                AND B.NUM_SEQ_HEADER = I.NUM_SEQ_HEADER
                AND B.NUM_SEQ_CORPO = I.NUM_SEQ_CORPO
                AND I.COD_ITEM_NL = P.CDARVPROD
                AND I.NUM_SEQ_HEADER = IMPICMS.NUM_SEQ_HEADER
                AND I.NUM_SEQ_CORPO = IMPICMS.NUM_SEQ_CORPO
                AND I.NUM_SEQ_ITEM = IMPICMS.NUM_SEQ_ITEM
                AND I.NUM_SEQ_HEADER = IMPCOFINS.NUM_SEQ_HEADER
                AND I.NUM_SEQ_CORPO = IMPCOFINS.NUM_SEQ_CORPO
                AND I.NUM_SEQ_ITEM = IMPCOFINS.NUM_SEQ_ITEM
                AND I.NUM_SEQ_HEADER = IMPPIS.NUM_SEQ_HEADER
                AND I.NUM_SEQ_CORPO = IMPPIS.NUM_SEQ_CORPO
                AND I.NUM_SEQ_ITEM = IMPPIS.NUM_SEQ_ITEM
        ORDER BY F.CDFILIALEXT, P.CDCLIENTE, B.NUM_NOTA, B.COD_SERIE, B.DTA_EMISSAO, H.NUM_RNF,
                I.COD_ITEM_NL, P.NMPRODUTO, I.VLR_ITEM
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelFil($params) {
        $params = [
            'P_CDOPERADOR' => $params['P_CDOPERADOR'],
            'P_CDFILIAL'   => $params['P_CDFILIAL']
        ];

        $sql = "
            SELECT DISTINCT F.CDFILIAL, NMFILIAL, F.CDFILIALEXT, CL.CDEXCLIENTE, CL.NMRAZSOCCLIE, CL.CDCLIENTE
                FROM FILIOPER FO
                    JOIN FILIAL F
                        ON F.CDFILIAL = FO.CDFILIAL
                    JOIN PARAVEND P 
                        ON F.CDFILIAL = P.CDFILIAL
                    LEFT JOIN CLIENTE CL
                        ON CL.CDCLIENTE = P.CDCLIENTE
                WHERE FO.CDOPERADOR = :P_CDOPERADOR
                    AND ((INSTR(:P_CDFILIAL, F.CDFILIAL) > 0) OR (:P_CDFILIAL = 'T'))
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelBody($params, $cdfilial) {
        $params = [
            'P_CDFILIAL'  => $cdfilial,
            'P_DTINICIAL' => $params['P_DTINICIAL'],
            'P_DTFINAL'   => $params['P_DTFINAL']
        ];

        $sql = "
            SELECT SUM(VRTOTVENDA) VRTOTVENDA, MAX(NRNOTAFISCALCE) NRNOTAFISC,
                    DECODE(C.IDTPEMISSAOFOS, 'FNC', '5214', 'SAT', '5215') CODOPER,
                    CL.CDEXCLIENTE NUMCPFCNPJ, CL.NMRAZSOCCLIE,
                    NVL(C.CDSERIECX, C.CDCAIXA) CDSERIE,
                    TRUNC(V.DTEMISSAONFCE) DTEMISSAONFCE,
                    F.NMFILIAL, V.DTENTRVENDA, NVL(V.NRHEADERINTSAP, 'XX') NRHEADERINTSAP
                FROM V_FILIALLOJACAIXA FLC
                    JOIN VENDA V
                        ON FLC.CDFILIAL = V.CDFILIAL
                            AND FLC.CDCAIXA = V.CDCAIXA
                            AND FLC.CDLOJA = V.CDLOJA
                    JOIN CAIXA C
                        ON FLC.CDFILIAL = C.CDFILIAL
                            AND FLC.CDCAIXA = C.CDCAIXA
                    JOIN FILIAL F 
                        ON FLC.CDFILIAL = F.CDFILIAL
                    JOIN PARAVEND P 
                        ON FLC.CDFILIAL = P.CDFILIAL
                    LEFT JOIN CLIENTE CL
                        ON P.CDCLIENTE = CL.CDCLIENTE
                WHERE FLC.CDFILIAL = :P_CDFILIAL
                    AND V.DTENTRVENDA BETWEEN :P_DTINICIAL AND :P_DTFINAL
                    AND V.IDSITUVENDA = 'O'
                GROUP BY DECODE(C.IDTPEMISSAOFOS, 'FNC', '5214', 'SAT', '5215'), CL.CDEXCLIENTE, CL.NMRAZSOCCLIE,
                        NVL(C.CDSERIECX, C.CDCAIXA), TRUNC(V.DTEMISSAONFCE),
                        F.NMFILIAL, V.DTENTRVENDA, NVL(V.NRHEADERINTSAP, 'XX')
            ORDER BY V.DTENTRVENDA, 2, NVL(C.CDSERIECX, C.CDCAIXA)
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelItens($relFil, $relBody) {
        $params = [
            'P_CDFILIAL'       => $relFil['CDFILIAL'],
            'P_DTENTRVENDA'    => $relBody['DTENTRVENDA'],
            'P_CDSERIE'        => $relBody['CDSERIE'],
            'P_NRHEADERINTSAP' => $relBody['NRHEADERINTSAP']
        ];

        $sql = "
            SELECT P.CDARVPROD, P.CDPRODINTE, P.NMPRODUTO, II.CDCFOPPROD, P.CDCLASFISC CDNCM, 
                    NVL(VRUNITVEND, 0) + NVL(I.VRUNITVENDCL, 0) VRUNIT, II.CDCSTPRODI,
                    SUM(NVL(I.VRDESITVEND, 0)) VRDESC, SUM(I.QTPRODVEND) QTDEITEM,
                    SUM(TRUNC((I.QTPRODVEND * (I.VRUNITVEND + NVL(I.VRUNITVENDCL, 0))) - NVL(I.VRDESITVEND, 0) + NVL(I.VRACRITVEND, 0), 2)) VRITEM
                FROM V_FILIALLOJACAIXA FLC
                    JOIN VENDA V
                        ON FLC.CDFILIAL = V.CDFILIAL
                            AND FLC.CDCAIXA = V.CDCAIXA
                            AND FLC.CDLOJA = V.CDLOJA
                    JOIN ITEMVENDA I
                        ON V.CDFILIAL = I.CDFILIAL
                            AND V.CDCAIXA = I.CDCAIXA
                            AND V.NRSEQVENDA = I.NRSEQVENDA
                    JOIN ITVENDAIMPOS II
                        ON II.CDFILIAL = I.CDFILIAL
                            AND II.CDCAIXA = I.CDCAIXA
                            AND II.NRSEQVENDA = I.NRSEQVENDA
                            AND II.NRSEQUITVEND = I.NRSEQUITVEND
                    JOIN CAIXA C
                        ON FLC.CDFILIAL = C.CDFILIAL
                            AND FLC.CDCAIXA = C.CDCAIXA
                            AND V.CDFILIAL = C.CDFILIAL
                            AND V.CDCAIXA = C.CDCAIXA
                    JOIN PRODUTO P
                        ON I.CDPRODUTO = P.CDPRODUTO
                WHERE FLC.CDFILIAL = :P_CDFILIAL
                    AND V.DTENTRVENDA = :P_DTENTRVENDA
                    AND NVL(C.CDSERIECX, C.CDCAIXA) = :P_CDSERIE
                    AND V.IDSITUVENDA = 'O'
                    AND NVL(V.NRHEADERINTSAP, 'XX') = :P_NRHEADERINTSAP
                GROUP BY P.CDARVPROD, P.CDPRODINTE, P.NMPRODUTO, II.CDCFOPPROD,
                    P.CDCLASFISC, NVL(VRUNITVEND, 0) + NVL(I.VRUNITVENDCL, 0), II.CDCSTPRODI
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqCodExtProd($cdarvprod) {
        $params = [
            'P_CDARVPROD' => $cdarvprod
        ];

        $sql = "
            SELECT 'PRODUTO ' || CDARVPROD || ' - ' || NMPRODUTO || ' SEM CODIGO EXTERNO PARAMETRIZADO.' AS VDPRODUTO
                FROM PRODUTO
                WHERE CDARVPROD = :P_CDARVPROD
                    AND CDPRODINTE IS NULL
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelNCM($cdarvprod) {
        $params = [
            'P_CDARVPROD' => $cdarvprod
        ];

        $sql = "
            SELECT 'NCM DO PRODUTO ' || P.CDARVPROD || ' - ' || P.NMPRODUTO || ' CONTEM PONTOS. O NCM ' || 'DEVE CONTER APENAS NUMEROS NO CADASTRO.' AS DSVALIDACAO
                FROM PRODUTO P
                WHERE P.CDARVPROD = :P_CDARVPROD
                    AND INSTR(P.CDCLASFISC, '.') > 0
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqTamNCM($cdarvprod) {
        $params = [
            'P_CDARVPROD' => $cdarvprod
        ];

        $sql = "
            SELECT 'NCM DO PRODUTO ' || P.CDARVPROD || ' - ' || P.NMPRODUTO || ' POSSUI TAMANHO INCORRETO.' AS DSVALIDACAO
                FROM PRODUTO P
                WHERE P.CDARVPROD = :P_CDARVPROD
                    AND (P.CDCLASFISC IS NULL OR LENGTH(P.CDCLASFISC) <> 8)
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelImposto($adqRelBody, $adqRelFil, $adqRelItens) {
        $params = [
            'P_CDFILIAL'       => $adqRelFil['CDFILIAL'],
            'P_DTENTRVENDA'    => $adqRelBody['DTENTRVENDA'],
            'P_CDCFOP'         => $adqRelItens['CDCFOPPROD'],
            'P_CDCST'          => $adqRelItens['CDCSTPRODI'],
            'P_CDNCM'          => $adqRelItens['CDNCM'],
            'P_CDSERIE'        => $adqRelBody['CDSERIE'],
            'P_CDARVPROD'      => $adqRelItens['CDARVPROD'],
            'P_VRUNIT'         => $adqRelItens['VRUNIT'],
            'P_NRHEADERINTSAP' => $adqRelBody['NRHEADERINTSAP']
        ];
        
        $sql = "
            SELECT II.CDCFOPPROD, SUM(II.VRIMPOPRODIT) VRIMPOPRODIT,
                    SUM(II.VRIMPPIS) VRIMPPIS, SUM(II.VRIMPCOFINS) VRIMPCOFINS
                FROM V_FILIALLOJACAIXA FLC
                    JOIN VENDA V
                        ON FLC.CDFILIAL = V.CDFILIAL
                            AND FLC.CDCAIXA = V.CDCAIXA
                            AND FLC.CDLOJA = V.CDLOJA
                    JOIN ITEMVENDA I
                        ON V.CDFILIAL = I.CDFILIAL
                            AND V.CDCAIXA = I.CDCAIXA
                            AND V.NRSEQVENDA = I.NRSEQVENDA
                    JOIN ITVENDAIMPOS II
                        ON II.CDFILIAL = I.CDFILIAL
                            AND II.CDCAIXA = I.CDCAIXA
                            AND II.NRSEQVENDA = I.NRSEQVENDA
                            AND II.NRSEQUITVEND = I.NRSEQUITVEND
                    JOIN CAIXA C
                        ON FLC.CDFILIAL = C.CDFILIAL
                            AND FLC.CDCAIXA = C.CDCAIXA
                            AND V.CDFILIAL = C.CDFILIAL
                            AND V.CDCAIXA = C.CDCAIXA
                    JOIN PRODUTO P
                        ON I.CDPRODUTO = P.CDPRODUTO
                WHERE FLC.CDFILIAL = :P_CDFILIAL
                    AND V.DTENTRVENDA = :P_DTENTRVENDA
                    AND II.CDCFOPPROD = :P_CDCFOP
                    AND II.CDCSTPRODI = :P_CDCST
                    AND P.CDCLASFISC = :P_CDNCM
                    AND NVL(C.CDSERIECX, C.CDCAIXA) = :P_CDSERIE
                    AND V.IDSITUVENDA = 'O'
                    AND P.CDARVPROD = :P_CDARVPROD
                    AND NVL(VRUNITVEND, 0) + NVL(I.VRUNITVENDCL, 0) = :P_VRUNIT
                    AND NVL(V.NRHEADERINTSAP, 'XX') = :P_NRHEADERINTSAP
                GROUP BY II.CDCFOPPROD
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelTipoDoc($adqRelItens) {
        $params = [
            'P_CDCFOP' => $adqRelItens['CDCFOPPROD'],
            'P_CDCST'  => $adqRelItens['CDCSTPRODI'],
            'P_CDNCM'  => $adqRelItens['CDNCM']
        ];

        $sql = "
            SELECT T.CDTPDOC
                FROM TPDOCINTSAPFOS T
                WHERE T.CDCFOP = :P_CDCFOP
                    AND T.CDCST = :P_CDCST
                    AND T.CDNCM = :P_CDNCM
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelPeriodo($relFil, $params) {
        $params = [
            'P_CDFILIAL'  => $relFil['CDFILIAL'],
            'P_DTINICIAL' => $params['P_DTINICIAL'],
            'P_DTFINAL'   => $params['P_DTFINAL']
        ];

        $sql = "
            SELECT DISTINCT 'EMPRESA ' ||E.CDEMPRESA || ' - ' || E.NMEMPRESA|| ' FORA DO PERIODO PERMITIDO PARA GERACAO DE RNF.' AS DSVALIDACAO
                FROM EMPRESA E
                    JOIN FILIAL F
                        ON F.CDEMPRESA = E.CDEMPRESA
                WHERE :P_CDFILIAL = F.CDFILIAL
                    AND NOT EXISTS (
                        SELECT 1
                            FROM PERIODORNF P
                            WHERE E.CDEMPRESA = P.CDEMPRESA
                                AND ((:P_DTINICIAL BETWEEN P.DTINICIAL AND P.DTFINAL) OR (:P_DTFINAL BETWEEN P.DTINICIAL AND P.DTFINAL))
                    )
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelCodExtFil($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT DISTINCT 'FILIAL ' || FI.CDFILIAL || ' - ' || FI.NMFILIAL || ' SEM CODIGO EXTERNO PARAMETRIZADO.' AS FILIALVD
                FROM FILIOPER FO, FILIAL FI
                WHERE FI.CDFILIAL = :P_CDFILIAL
                    AND FI.CDFILIALEXT IS NULL
                    AND EXISTS (
                        SELECT 1
                            FROM CAIXA CX
                                JOIN CAIXAPERS CP
                                    ON CX.CDFILIAL = CP.CDFILIAL
                                        AND CX.CDCAIXA = CP.CDCAIXAPERS
                            WHERE CX.CDFILIAL = FI.CDFILIAL
                                AND CX.IDCOLETOR IN ('R', 'N')
                    )
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelTamCodFil($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT DISTINCT 'FILIAL '|| FI.CDFILIAL || ' - ' || FI.NMFILIAL || ' COM CODIGO EXTERNO MAIOR QUE 5 CARACTERES.' AS FILIALVD
                FROM FILIAL FI
                WHERE FI.CDFILIAL = :P_CDFILIAL
                    AND LENGTH(FI.CDFILIALEXT) > 5
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelCodExtEmpr($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT 'EMPRESA ' || E.CDEMPRESA || E.NMEMPRESA|| ' SEM CODIGO EXTERNO PARAMETRIZADO.' AS DSVALIDACAO
                FROM EMPRESA E
                    JOIN FILIAL F
                        ON F.CDEMPRESA = E.CDEMPRESA
                WHERE :P_CDFILIAL = F.CDFILIAL
                    AND E.CDEXTERNO IS NULL
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelConvFil($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT 'FILIAL ' || CDFILIAL || ' - ' || NMFILIAL || ' SEM CODIGO DE CONVERSAO PARAMETRIZADO.' AS VDFILIAL
                FROM (
                    SELECT F.CDFILIAL, F.NMFILIAL, F.CDFILIALEXT
                        FROM FILIAL F
                        WHERE F.CDFILIAL = :P_CDFILIAL
                            AND LENGTH(CDFILIALEXT) <= 5
                            AND EXISTS (
                                SELECT 1
                                    FROM CAIXA CX
                                        JOIN CAIXAPERS CP
                                            ON CX.CDFILIAL = CP.CDFILIAL
                                                AND CX.CDCAIXA = CP.CDCAIXAPERS
                                    WHERE CX.CDFILIAL = F.CDFILIAL
                                        AND CX.IDCOLETOR IN ('R', 'N')
                            )
                ) F
                WHERE FILIALSAPRNF(CDFILIAL) IS NULL
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelClienPadrao($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT 'FILIAL ' || F.CDFILIAL || ' - ' || F.NMFILIAL || ' SEM CLIENTE PADRAO PARAMETRIZADO.' AS VDFILIAL
                FROM PARAVEND P
                    JOIN FILIAL F
                        ON P.CDFILIAL = F.CDFILIAL
                WHERE F.CDFILIAL = :P_CDFILIAL
                    AND P.CDCLIENTE IS NULL
                    AND EXISTS (
                        SELECT 1
                            FROM CAIXA CX
                                JOIN CAIXAPERS CP
                                    ON CX.CDFILIAL = CP.CDFILIAL
                                        AND CX.CDCAIXA = CP.CDCAIXAPERS
                            WHERE CX.CDFILIAL = P.CDFILIAL
                                AND CX.IDCOLETOR IN ('R', 'N')
                    )
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelCodExtClien($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT 'CLIENTE PADRAO (' || CL.CDCLIENTE || ' - ' || CL.NMRAZSOCCLIE || ')' || ' RELACIONADO A FILIAL ' || P.CDFILIAL || ' NAO POSSUI CODIGO EXTERNO PARAMETRIZADO.' VDFILIAL
                FROM PARAVEND P 
                    LEFT JOIN CLIENTE CL
                        ON P.CDCLIENTE = CL.CDCLIENTE
                WHERE :P_CDFILIAL = P.CDFILIAL
                    AND CL.CDEXCLIENTE IS NULL
                    AND EXISTS (
                        SELECT 1
                            FROM CAIXA CX
                                JOIN CAIXAPERS CP
                                    ON CX.CDFILIAL = CP.CDFILIAL
                                        AND CX.CDCAIXA = CP.CDCAIXAPERS
                            WHERE CX.CDFILIAL = P.CDFILIAL
                                AND CX.IDCOLETOR IN ('R', 'N')
                    )
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelTipoEmis($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT DISTINCT 'FILIAL ' || FO.CDFILIAL || ' - ' || FO.NMFILIAL || ' POSSUI CAIXAS COM MAIS DE UM TIPO DE EMISSAO. FAVOR CORRIGIR ESSA SITUACAO.' AS VDFILIAL
                FROM CAIXA C
                    JOIN FILIAL FO
                        ON C.CDFILIAL = FO.CDFILIAL
                    JOIN CAIXAPERS CP
                        ON C.CDFILIAL = CP.CDFILIAL
                            AND C.CDCAIXA = CP.CDCAIXAPERS
                WHERE FO.CDFILIAL = :P_CDFILIAL
                    AND C.IDCOLETOR IN ('R', 'N')
                    AND EXISTS (
                        SELECT 1
                            FROM CAIXA CX
                                JOIN CAIXAPERS CPS
                                    ON CX.CDFILIAL = CPS.CDFILIAL
                                        AND CX.CDCAIXA = CPS.CDCAIXAPERS
                            WHERE CX.CDFILIAL = C.CDFILIAL
                                AND CX.IDTPEMISSAOFOS <> C.IDTPEMISSAOFOS
                                AND CX.IDCOLETOR IN ('R', 'N')
                    )
            ORDER BY 1
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqRelTamCodExtEmpr($relFil) {
        $params = [
            'P_CDFILIAL' => $relFil['CDFILIAL']
        ];

        $sql = "
            SELECT 'CODIGO EXTERNO DA EMPRESA ' || E.CDEMPRESA || ' - ' || E.NMEMPRESA || ' MAIOR QUE 4 CARACTERES.' AS VDEMPRESA
                FROM EMPRESA E
                    JOIN FILIAL F
                        ON F.CDEMPRESA = E.CDEMPRESA
                WHERE F.CDFILIAL = :P_CDFILIAL
                    AND LENGTH(E.CDEXTERNO) > 4
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqMaxDtAbert($relFil, $params) {
        $params = [
            'P_CDFILIAL'   => $relFil['CDFILIAL'],
            'P_CDOPERADOR' => $params['P_CDOPERADOR'],
            'P_DTINICIAL'  => $params['P_DTINICIAL'],
            'P_DTFINAL'    => $params['P_DTFINAL']
        ];

        $sql = "
            SELECT T.CDFILIAL, T.CDCAIXA, TO_CHAR(DTABERCAIX, 'DD/MM/YYYY') AS DIA, MAX(DTABERCAIX) AS DTABERCAIX
                FROM TURCAIXA T
                    JOIN (
                        SELECT DISTINCT F.CDFILIAL, F.NMFILIAL
                            FROM V_FILIALLOJACAIXA F
                                JOIN FILIOPER FO
                                    ON F.CDFILIAL = FO.CDFILIAL
                            WHERE ((INSTR(:P_CDFILIAL, F.CDFILIAL) > 0) OR (:P_CDFILIAL = 'T'))
                            AND FO.CDOPERADOR = :P_CDOPERADOR
                    ) FO
                        ON T.CDFILIAL = FO.CDFILIAL
                    JOIN CAIXA C
                        ON T.CDFILIAL = C.CDFILIAL
                            AND T.CDCAIXA = C.CDCAIXA
                WHERE C.IDCOLETOR IN ('N', 'R')
                    AND TRUNC(DTABERCAIX) BETWEEN :P_DTINICIAL AND :P_DTFINAL
                GROUP BY T.CDFILIAL, T.CDCAIXA, TO_CHAR(DTABERCAIX, 'DD/MM/YYYY')
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    public function adqChecaFechamento($maxDtAbert) {
        $params = [
            'P_CDFILIAL'   => $maxDtAbert['CDFILIAL'],
            'P_CDCAIXA'    => $maxDtAbert['CDCAIXA'],
            'P_DTABERCAIX' => $maxDtAbert['DTABERCAIX'],
            'P_DTDIA'      => $maxDtAbert['DIA']
        ];

        $sql = "
            SELECT 'CAIXA ' || :P_CDCAIXA || ' DA FILIAL ' || :P_CDFILIAL || ' NAO POSSUI O ULTIMO FECHAMENTO DO DIA ' || :P_DTDIA AS DSVALIDACAO
                FROM TURCAIXA
                WHERE CDFILIAL = :P_CDFILIAL
                    AND CDCAIXA = :P_CDCAIXA
                    AND DTABERCAIX = :P_DTABERCAIX
                    AND IDULTFECHAMEN <> 'S'
        ";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
}